<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://rankfortress.com/
 * @since      1.0.0
 *
 * @package    Location_Geocentric
 * @subpackage Location_Geocentric/includes
 */

class Location_Geocentric {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Location_Geocentric_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'LOCATION_GEOCENTRIC_VERSION' ) ) {
			$this->version = LOCATION_GEOCENTRIC_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'location-geocentric';

		$this->load_dependencies();
		$this->set_locale();
        $this->define_admin_hooks();

        $appsero = new Appsero();

        if( $appsero->appsero_check_license() ){
		    $this->define_public_hooks();
            $this->shortcodes();
        }
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Location_Geocentric_Loader. Orchestrates the hooks of the plugin.
	 * - Location_Geocentric_i18n. Defines internationalization functionality.
	 * - Location_Geocentric_Admin. Defines all hooks for the admin area.
	 * - Location_Geocentric_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-location-geocentric-loader.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-location-geocentric-appsero.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-location-geocentric-i18n.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-location-geocentric-settings.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/settings/fields-design.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/settings/fields-settings.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-location-geocentric-shortcodes.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-location-geocentric-admin.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-location-geocentric-public.php';

		$this->loader = new Location_Geocentric_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Location_Geocentric_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Location_Geocentric_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Location_Geocentric_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

	}

    private function shortcodes() {
        
        $shortcode_admin = new Location_Geocentric_Shortcodes( $this->get_plugin_name(), $this->get_version() );
        add_shortcode( 'rkfrt-neighborhoods', array($shortcode_admin, 'neighborhood') );
        add_shortcode( 'rkfrt-activities', array($shortcode_admin, 'activities') );
        add_shortcode( 'rkfrt-map', array($shortcode_admin, 'map') );
        add_shortcode( 'rkfrt-details', array($shortcode_admin, 'about_location') );
        add_shortcode( 'rkfrt-directions', array($shortcode_admin, 'directions') );
        add_shortcode( 'rkfrt-reviews', array($shortcode_admin, 'reviews') );
    }

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Location_Geocentric_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
        $this->loader->add_action( 'wp_head', $plugin_public, 'header_meta' );
        

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Location_Geocentric_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

    

}